#ifndef pqOptAirfoil_h
#define pqOptAirfoil_h

#include "pqPropertyWidget.h"

#include <iostream>
#include <cstdlib>
#include <cstring>
#include <sstream>
#include <string>
#include <vector>

#include <QWidget>
#include <QString>
#include <QLineEdit>
#include <QComboBox>
#include <QString>

class pqOptAirfoil : public pqPropertyWidget
{
  Q_OBJECT
  typedef pqPropertyWidget Superclass;

public:
  pqOptAirfoil(
    vtkSMProxy* smproxy, vtkSMProperty* smproperty, QWidget* parentObject = 0);
  virtual ~pqOptAirfoil();

protected:

  bool* goodDocker;

  bool* preProcessingComplete;

  bool* meshGenComplete;

  bool* processingComplete;

  bool* profileReferenced;

  bool* advancedSettingsEnabled;

  bool *advancedTurbulenceEnabled;

  bool *advancedRunscriptEnabled;

  bool *optimizationRunning;
  
  double* chordValue;

  double* scaleFactor;

  double* originalChordValue;

  double* profileMinY;
  
  double* profileMaxY;

  double* nAirplaneSurfaceMeshCells;

  double* prtValue;

  double* referenceAreaValue;

  double* alphaZeroValue;

  double zWidth;

  int* printIntervalValue;

  QLineEdit* nTotalCells;

  QLineEdit* yWall;

  QLineEdit* NpExtrude;

  QLineEdit* marchingDistanceValue;

  QLineEdit* dX1PSValue;

  // QLineEdit* Alpha1PSValue;

  QLineEdit* dX2PSValue;

  // QLineEdit* Alpha2PSValue;

  QLineEdit* dXMaxPSValue;

  QLineEdit* dX1SSValue;

  // QLineEdit* Alpha1SSValue;

  QLineEdit* dX2SSValue;

  // QLineEdit* Alpha2SSValue;

  QLineEdit* dXMaxSSValue;

  QLineEdit* NpTEValue;

  QString* caseFileDir;

  std::string preProcessingFile;

  std::string rootPath;

  std::string lowerPath;

  std::string upperPath;

  std::string logMeshGenOut;
  
  std::string ffdPtsFile;

  std::string* optOutputFile;

  std::string* pyGuiPath;

  std::string* alphaTitle;
  
  std::string* optTitle;

  std::string* simTitle;

  std::string* daVersion;

  std::string solverName;

  QString* upperSurfacePath;

  QString* lowerSurfacePath;
  
  QLineEdit* machNumberValue;
  
  QLineEdit* reynoldsNumberValue;

  QLineEdit* dynamicViscosityValue;

  QLineEdit* alphaValue;

  QLineEdit* temperatureValue;

  QLineEdit* farFieldPressureValue;

  QLineEdit* normalizationDensityValue;

  QLineEdit* eddyViscosityRatioValue;

  QLineEdit* turbulentIntensityValue;

  QLineEdit* targetLiftCoefficientValue;

  QLineEdit* flowConvergenceToleranceValue;

  QLineEdit* adjointConvergenceToleranceValue;

  QLineEdit* numberFFDPointsValue;

  QLineEdit* optimalityTolerance;

  QLineEdit* feasibilityTolerance;

  QLineEdit* minimumThicknessValue;

  QLineEdit* maximumThicknessValue;

  QLineEdit* minimumVolumeValue;

  QLineEdit* maximumVolumeValue;

  QLineEdit* maximumIterationsValue;

  QLineEdit* flowIterationsValue;

  QLineEdit* customDaConfigValue;
  
  QComboBox* turbulenceModelBox;
  
  QComboBox* solverBox;

  QComboBox* optimizerBox;

  QComboBox* runType;

protected slots:
  void toggleAdvancedSettings();
  void runTypeChanged();
  void dockerMessage();
  void aboutMessage();
  void caseFileHandler();
  void calculateAdvancedMeshSettings();
  void calculateTotalCells();
  void preProcessing();
  void genMesh();
  void selectUpperSurface();
  void selectLowerSurface();
  void writeGenMeshScript();
  void machWarning();
  void toggleAdvancedTurbulence();
  void toggleAdvancedRunscript();
  void advancedRunscript();
  void writeConstant();
  void writeOrig();
  void writeSystem();
  void writeRunScript();
  void caseSpecificParameters();
  void scaleMesh();
  void scaleReynolds();
  void genFFDPoints();
  void scaleFFDPoints();
  void getReferenceChord();
  void postProcessHandler();


private:
  Q_DISABLE_COPY(pqOptAirfoil)
};

#endif
