#ifndef pqOptUMesh_h
#define pqOptUMesh_h

#include "pqPropertyWidget.h"

#include <iostream>
#include <cstdlib>
#include <cstring>
#include <sstream>
#include <string>
#include <vector>

#include <QWidget>
#include <QString>
#include <QLineEdit>
#include <QComboBox>
#include <QTableWidget>

//-----------------------------------------------------------------------------

class pqOptUMesh : public pqPropertyWidget
{
  Q_OBJECT
  typedef pqPropertyWidget Superclass;

public:
  pqOptUMesh(
    vtkSMProxy* smproxy, vtkSMProperty* smproperty, QWidget* parentObject = 0);
  virtual ~pqOptUMesh();

protected:
  bool* meshGenComplete;
  bool* goodDocker;
  bool* stl;
  bool* advancedSnappyEnabled;

  int flowIterationsValue;

  std::string rootPath;
  std::string triSurfacePath;
  std::string preProcessFile;
  std::string meshGenLog;
  std::string dafoamVersion;
  std::string* pyGuiPath;

  std::vector<float> *meshLocation;

  QString* caseFileDir;
  QStringList surfacePaths;

  QLineEdit *blockMeshXMin, *blockMeshXMax, *blockMeshYMin, *blockMeshYMax, *blockMeshZMin, *blockMeshZMax, *blockMeshTotalCells;
  QLineEdit *lineLevelValue, *surfaceLevelValue, *prismLevelValue, *cellsBetweenLevelsValue, *finalLayerThicknessValue, *maxNonOrthoValue, *maxSkewnessValue;

  std::vector<QLineEdit*> *advancedLineLevels, *advancedSurfaceLevels;

  QComboBox *meshLocationValue;

  std::vector<QComboBox*> *blockBoundaryTypes;

  QTableWidget *advancedLevelValues, *blockBoundaryTable;

protected slots:
  void aboutMessage();
  void caseFileHandler();
  void dockerMessage();
  void selectSurface();
  void visualizeBackgroundMesh();
  bool checkBlock();
  bool checkLocation();
  bool checkUnset();
  void toggleAdvancedSnappy();
  void setLineEdits();
  void changeLimits();
  void getLocation();
  std::string writePreProcessing();
  void controlDict(std::string);
  void fvSchemes(std::string);
  void fvSolution(std::string);
  int blockMeshDict(std::string);
  void surfaceFeatureExtractDict(std::string);
  void snappyHexMeshDict(std::string);
  int generateMesh();
  //int checkMesh();
  //int transformPoints();

private:
  Q_DISABLE_COPY(pqOptUMesh)
};

#endif
